//! [![pipeline status](https://gitlab.com/mexus/sedregex/badges/master/pipeline.svg)](https://gitlab.com/mexus/sedregex/commits/master)
//! [![crates.io](https://img.shields.io/crates/v/sedregex.svg)](https://crates.io/crates/sedregex)
//! [![docs.rs](https://docs.rs/sedregex/badge.svg)](https://docs.rs/sedregex)
//!
//! [[Release docs]](https://docs.rs/sedregex/)
//!
//! [[Master docs]](https://mexus.gitlab.io/sedregex/sedregex/)
//!
//! A simple sed-like library that uses regex under the hood.
//!
//! ## License
//!
//! Licensed under either of
//!
//!  * Apache License, Version 2.0 ([LICENSE-APACHE](LICENSE-APACHE) or http://www.apache.org/licenses/LICENSE-2.0)
//!  * MIT license ([LICENSE-MIT](LICENSE-MIT) or http://opensource.org/licenses/MIT)
//!
//! at your option.
//!
//! ### Contribution
//!
//! Unless you explicitly state otherwise, any contribution intentionally submitted
//! for inclusion in the work by you, as defined in the Apache-2.0 license, shall be dual licensed as above, without any
//! additional terms or conditions.

#![deny(missing_docs)]

extern crate regex;

mod command;
mod regex_flags;
mod splitting;
mod splitting_iter;

pub use command::Command;
pub use regex_flags::RegexFlag;

use splitting::split_for_replace;
use std::borrow::Cow;

/// An error that might happen during the parsing.
#[derive(Debug, PartialEq)]
pub enum ErrorKind {
    /// A given string has not enough segments.
    NotEnoughSegments,
    /// Unknown regex command has been detected.
    UnknownCommand(String),
    /// Unknown regex flags has been detected.
    UnknownFlag(char),
    /// Regex parsing/compiling error.
    RegexError(regex::Error),
}

/// Parses and executed a regex replace command in a form of `s/regex/replace/flags`.
///
/// # Usage notes
///
/// * Slashes (`/`) could be escaped by a backslash: `\/`
/// * For the list of supported commands please refer to the [`Command`](enum.Command.html) enum
/// * For the list of supported flags please refer to the [`RegexFlag`](enum.RegexFlag.html) enum
///
/// ```
/// extern crate sedregex;
/// use sedregex::find_and_replace;
///
/// fn main() {
///     // Both case-insensitive and global:
///     assert_eq!(
///         "Please stop wuts oh wut.",
///         find_and_replace("s/lol/wut/ig", "Please stop Lols oh lol.").unwrap()
///     );
///
///     // Same, but skipping the `s` character.
///     assert_eq!(
///         "Please stop wuts oh wut.",
///         find_and_replace("/lol/wut/ig", "Please stop Lols oh lol.").unwrap()
///     );
///
///     // Skipping the flags.
///     assert_eq!(
///         "Please stop Lols oh wut.",
///         find_and_replace("/lol/wut/", "Please stop Lols oh lol.").unwrap()
///     );
///
///     // Skipping the last slash.
///     assert_eq!(
///         "Please stop Lols oh wut.",
///         find_and_replace("/lol/wut", "Please stop Lols oh lol.").unwrap()
///     );
///
///     // Escaping a slash in a replace part.
///     assert_eq!(
///         r"Please stop wut/s oh wut/.",
///         find_and_replace(r"s/lol/wut\//gi", "Please stop Lols oh lol.").unwrap()
///     );
///
///     // Some regex stuff. Note that slashes meant for the regex are escaped.
///     // Also note the lack of the trailing slash: it's opitonal!
///     assert_eq!(
///         "Second, First",
///         find_and_replace(
///             r"s/(?P<first>[^,\\s]+),\\s+(?P<last>\\S+)/$last, $first",
///             "First, Second").unwrap()
///     );
///
///     // Ok let's go with some simple regex stuff.
///     assert_eq!(
///         "Some weird typo",
///         find_and_replace(r"s/ie/ei/", "Some wierd typo").unwrap()
///     );
/// }
/// ```
pub fn find_and_replace<'a>(command: &str, text: &'a str) -> Result<Cow<'a, str>, ErrorKind> {
    let data = split_for_replace(command)?;
    let regex = data.build_regex()?;
    if data.flags.is_global() {
        Ok(regex.replace_all(text, data.with.as_ref()))
    } else {
        Ok(regex.replace(text, data.with.as_ref()))
    }
}
