use std::borrow::Cow;

/// Splits a given string at a slash (`/`), skipping first `begin` chars and eating away
/// escapes (`/`).
///
/// If the slash was found, the function returns a part of string after the `begin` and before
/// the slash (slash not included) and the position of the slash.
///
/// If the slash was not found, the whole string starting from the `begin` is returned (with
/// escaped slashes being unescaped), and the position equals to the string length.
fn split_escape<'a>(string: &'a str, begin: usize) -> (Cow<'a, str>, usize) {
    enum State {
        Borrowed,
        Owned(String),
        Escape(String),
    }
    let mut state = State::Borrowed;
    for (idx, c) in string.char_indices().skip(begin) {
        state = match state {
            State::Escape(mut buf) => {
                buf.push(c);
                State::Owned(buf)
            }
            State::Borrowed => match c {
                '\\' => State::Escape(String::from(&string[begin..idx])),
                '/' => {
                    return (Cow::Borrowed(&string[begin..idx]), idx);
                }
                _ => State::Borrowed,
            },
            State::Owned(mut buf) => match c {
                '\\' => State::Escape(buf),
                '/' => {
                    return (Cow::Owned(buf), idx);
                }
                c => {
                    buf.push(c);
                    State::Owned(buf)
                }
            },
        }
    }
    match state {
        State::Borrowed => (Cow::Borrowed(&string[begin..]), string.len()),
        State::Owned(buf) | State::Escape(buf) => (Cow::Owned(buf), string.len()),
    }
}

/// An iterator over splitted parts of a string.
pub struct SlashSplitIter<'a> {
    original: &'a str,
    position: usize,
}

impl<'a> From<&'a str> for SlashSplitIter<'a> {
    fn from(string: &'a str) -> Self {
        SlashSplitIter {
            original: string,
            position: 0,
        }
    }
}

impl<'a> Iterator for SlashSplitIter<'a> {
    type Item = Cow<'a, str>;

    fn next(&mut self) -> Option<Self::Item> {
        if self.position >= self.original.len() {
            return None;
        }
        let (part, pos) = split_escape(self.original, self.position);
        self.position = match pos.checked_add(1) {
            Some(x) => x,
            None => self.original.len(),
        };
        Some(part)
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn split_first_empty() {
        let data = "/something/";
        assert_eq!(("".into(), 0), split_escape(data, 0));
    }

    #[test]
    fn split_once() {
        let data = r"s/ahaha\/lol/wut/g";
        assert_eq!(("s".into(), 1), split_escape(data, 0));
        assert_eq!(("".into(), 1), split_escape(data, 1));
        assert_eq!(("ahaha/lol".into(), 12), split_escape(data, 2));
        assert_eq!(("haha/lol".into(), 12), split_escape(data, 3));
        assert_eq!(("".into(), 16), split_escape(data, 16));
        assert_eq!(("g".into(), 18), split_escape(data, 17));
    }

    #[test]
    fn split_once_last() {
        assert_eq!(("last/".into(), 6), split_escape(r"last\//", 0));
    }

    #[test]
    fn iter() {
        let data = r"a/fir\/st/second/th\/ird/fo\/rth/last";
        let mut iter = SlashSplitIter::from(data);
        assert_eq!(Some("a".into()), iter.next());
        assert_eq!(Some("fir/st".into()), iter.next());
        assert_eq!(Some("second".into()), iter.next());
        assert_eq!(Some("th/ird".into()), iter.next());
        assert_eq!(Some("fo/rth".into()), iter.next());
        assert_eq!(Some("last".into()), iter.next());
        assert_eq!(None, iter.next());
    }

    #[test]
    fn iter_first_empty() {
        let data = r"/fir\/st/second/th\/ird/fo\/rth/last";
        let mut iter = SlashSplitIter::from(data);
        assert_eq!(Some("".into()), iter.next());
        assert_eq!(Some("fir/st".into()), iter.next());
        assert_eq!(Some("second".into()), iter.next());
        assert_eq!(Some("th/ird".into()), iter.next());
        assert_eq!(Some("fo/rth".into()), iter.next());
        assert_eq!(Some("last".into()), iter.next());
        assert_eq!(None, iter.next());
    }

    #[test]
    fn iter_last_empty() {
        let data = "something/last/";
        let mut iter = SlashSplitIter::from(data);
        assert_eq!(Some("something".into()), iter.next());
        assert_eq!(Some("last".into()), iter.next());
        assert_eq!(None, iter.next());

        let data = r"something/last\/";
        let mut iter = SlashSplitIter::from(data);
        assert_eq!(Some("something".into()), iter.next());
        assert_eq!(Some("last/".into()), iter.next());
        assert_eq!(None, iter.next());
    }
}
